<?php
/**
 * Enquir3 Ajax feedback retrieval.
 */

/**
 * Class: EQ3F_Feedback_Ajax
 */
class EQ3F_Feedback_Ajax {

	/**
	 * Instance of feedback client.
	 *
	 * @var RemstoFeedbackClient $feedback_client
	 */
	protected $feedback_client;

	/**
	 * Constructor
	 *
	 * @param type $feedback_client Feedback client.
	 */
	public function __construct( $feedback_client ) {
		$this->feedback_client = $feedback_client;
	}

	/**
	 * Get data
	 *
	 * @return string Data to display
	 */
	public function get_data() {
		$post    = filter_input_array( INPUT_POST );
		$options = $post['options'];
		switch ( $options['type'] ) {
			case 'feedback_popup':
				$output = $this->get_popup_data( $options['id'] );
				break;
			case 'resend_mail':
				$fid          = $options['id'];
				$address_list = $options['al'];
				$output       = $this->send_email( $fid, $address_list );
				break;
			default:
				$output = 'Not Found';
				break;
		}
		return $output;
	}

	/**
	 * Display data for more pop up
	 *
	 * @param int $id Feedback ID.
	 * @return String Formated HTML mark for pop-up display.
	 */
	private function get_popup_data( $id ) {

		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';
		wp_enqueue_style( 'eq3f-base-style' );
		wp_enqueue_style( 'eq3f-item-style' );
		wp_enqueue_style( 'eq3f-icon-style' );

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$feedback         = $this->feedback_client->getFeedbackItem( null, null, $id );

		$name = array(
			'anon'       => $feedback->anonymous,
			'conc'       => $feedback->concealed,
			'first_name' => $feedback->first_name,
			'last_name'  => $feedback->last_name,
		);

		$author = RemstoModuleFunctions::feedback_name_display( $name );

		$rating = RemstoModuleFunctions::calculate_rating( $feedback );
		if ( false === $rating ) {
			$rating_summary = 'n/a';
		} else {
			$rating_summary = RemstoModuleFunctions::feedback_rating_display(
				array(
					'rating' => $rating,
					'label'  => '',
					'stars'  => 5,
				)
			);
		}

		if ( '' !== $feedback->image_src ) {
			$image_src = $feedback_options['remote_url'] . '/images/' . $feedback->image_src;
		} else {
			$image_src = '';
		}

		$popup_data = array(
			'stood_out'      => RemstoModuleFunctions::truncate_utf8(
				sanitize_textarea_field( $feedback->stood_out ),
				$feedback_options['feedback_display_stood_out_length'],
				true,
				true
			),
			'rating_summary' => $rating_summary,
			'title'          => $feedback->title ? sanitize_text_field( $feedback->title ) : '',
			'image_src'      => $image_src,
			'image_alt'      => $feedback->image_alt ? sanitize_text_field( $feedback->image_alt ) : '',
		);

		$popup_data['date'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Date:',
				'value' => gmdate( 'd/m/Y', $feedback->created ),
			)
		);

		$popup_data['first_name'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Name:',
				'value' => $author,
			)
		);

		$popup_data['company_comment'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => $feedback_options['reply_label'],
				'value' => $feedback->company_comment,
				'class' => 'tpp-field-company-comment',
			)
		);

		$popup_data['town'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Town:',
				'value' => $feedback->town,
			)
		);

		$popup_data['company'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Company:',
				'value' => $feedback->company_name,
			)
		);

		$popup_data['business_type'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Sector:',
				'value' => $feedback->business_category,
			)
		);

		if ( isset( $feedback->website ) && strlen( $feedback->website ) > 0 && '-' !== $feedback->website ) {
			$popup_data['website'] = RemstoModuleFunctions::iconWebLink( $feedback->website );
		}

		// Taxonomy.
		if ( isset( $feedback->taxonomy ) ) {
			$services = array();
			foreach ( $feedback->taxonomy as $tid => $taxonomy ) {
				$taxonomy = $this->feedback_client->getTaxonomy( $tid );
				if ( RemstoClientTaxonomyDisplayOptions::isOptionSet( $taxonomy->display_options, RemstoClientTaxonomyDisplayOptions::LISTSERVICE ) ) {
					foreach ( $feedback->taxonomy->$tid as $term_id ) {
						$term       = $taxonomy->terms->$term_id;
						$services[] = $term->title;
					}
				}
			}

			$services_text          = implode( ', ', $services );
			$popup_data['services'] = RemstoModuleFunctions::feedback_icon_text_value(
				array(
					'icon_class' => 'icon-services',
					'value'      => "<b>Service(s) Provided:</b> $services_text ",
				)
			);
		}

		if ( ! empty( $feedback->video_testimonial ) ) {
			$popup_data['video_testimonial'] = RemstoModuleFunctions::processVideoTestimonial( $feedback->id, $feedback->video_testimonial );
		}

		return EQ3F_Feedback::compile_template(
			EQ3F_PLUGIN_PATH . 'includes/views/feedback-tpp-entry.tpl.php',
			$popup_data
		);
	}


	/**
	 * Re-send new feedback email
	 *
	 * @param int    $fid Feedback ID.
	 * @param String $email_to Comma separated list of email addresses to send to.
	 * @return str Status message
	 */
	private function send_email( $fid, $email_to ) {
		if ( ! empty( $email_to ) ) {
			$options = get_option( 'enquir3_feedback_options_new', array() );
			$feedback_data    = (array) $this->feedback_client->getFeedbackItem( 0, 0, $fid );
			$email_from       = $options['feedback_email_notification_from'];
			$email_subject    = $options['feedback_email_notification_subject'] . 'EQ3_From:' . $email_from;

			// Convert taxonomy into a suitable array.
			if ( isset( $feedback_data['taxonomy'] ) ) {
				$taxonomy = array();
				foreach ( $feedback_data['taxonomy'] as $vocabulary_id => $terms ) {
					$taxonomy[ $vocabulary_id ] = array();
					foreach ( $terms as $term_id ) {
						if ( 0 !== $term_id ) {
							$taxonomy[ $vocabulary_id ][] = $term_id;
						}
					}
				}
				$feedback_data['taxonomy'] = $taxonomy;
			}

			// Service taxonomy.
			$tax_service                           = $this->feedback_client->getTaxonomy( $options['feedback_service_taxonomy'] );
			$feedback_data['service_taxonomy_all'] = json_decode( wp_json_encode( $tax_service ), true ); // casting does not convert nested objects.

			// Additional Taxonomy.
			$feedback_data['feedback_additional_taxonomy_all'] = array();
			foreach ( $options['feedback_additional_taxonomy'] as $vocabulary_id ) {
				$tax_additional                                      = $this->feedback_client->getTaxonomy( $vocabulary_id );
				$feedback_data['feedback_additional_taxonomy_all'][] = json_decode( wp_json_encode( $tax_additional ), true );
			}

			$feedback_data['options'] = $options;

			require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';

			$email_body = EQ3F_Feedback::compile_template(
				EQ3F_PLUGIN_PATH . 'views/remsto-notification-feedback-html-mail.tpl.php',
				$feedback_data
			);
			$headers    = array( 'Content-Type: text/html; charset=UTF-8' );
			wp_mail( $email_to, $email_subject, $email_body, $headers );
			return "$email_to - $email_subject - $email_body";
		} else {
			return 'No email addresses';
		}

	}
}
